"use strict";

// Fix back button cache problem
window.onunload = function () {};

var html = document.querySelector("html");

// Set the theme before any content is loaded, prevents flash
let theme;
try {
    theme = localStorage.getItem("mdbook-theme");
    // Work around some values being stored in localStorage wrapped in quotes
    if (theme.startsWith('"') && theme.endsWith('"')) {
        localStorage.setItem("mdbook-theme", theme.slice(1, theme.length - 1));
    }
} catch (e) {}
if (theme === null || theme === undefined) {
    theme = default_theme;
}
html.classList.remove("no-js");
html.classList.remove(default_theme);
html.classList.add(theme);
html.classList.add("js");

const sidebar_aria = () => {
    try {
        let sidebar = localStorage.getItem("mdbook-sidebar");
        // Work around some values being stored in localStorage wrapped in quotes
        if (sidebar.startsWith('"') && sidebar.endsWith('"')) {
            localStorage.setItem(
                "mdbook-sidebar",
                sidebar.slice(1, sidebar.length - 1)
            );
        }
    } catch (e) {}
    // Apply ARIA attributes after the sidebar and the sidebar toggle button are added to the DOM
    document
        .getElementById("sidebar-toggle")
        .setAttribute("aria-expanded", sidebar === "visible");
    document
        .getElementById("sidebar")
        .setAttribute("aria-hidden", sidebar !== "visible");
    Array.from(document.querySelectorAll("#sidebar a")).forEach(function (
        link
    ) {
        link.setAttribute("tabIndex", sidebar === "visible" ? 0 : -1);
    });
};

const configure_sidebar = () => {
    // Hide / unhide sidebar before it is displayed
    let sidebar = "hidden";
    try {
        sidebar = localStorage.getItem("mdbook-sidebar");
    } catch (e) {}
    sidebar = sidebar || "visible";
    html.classList.remove("sidebar-visible");
    html.classList.add("sidebar-" + sidebar);
};

configure_sidebar();

const create_ambient = async () => {
    const html = document.querySelector("html");
    const x = [];
    const y = [];
    const r = [];
    const x_speed = [];
    const y_speed = [];
    const r_speed = [];
    let src = "https://static.igem.wiki/teams/4161/wiki/ambient1.png";
    const add_particle = () => {
        x.push(Math.random() * window.innerWidth);
        y.push(Math.random() * window.innerHeight);
        r.push(Math.random() * 2 - 1);
        x_speed.push(
            (Math.random() * window.innerWidth * 2 - window.innerWidth) / 1000
        );
        y_speed.push(
            (Math.random() * window.innerHeight * 2 - window.innerHeight) / 1000
        );
        r_speed.push(Math.random() * 2 - 1);
        const particle = document.createElement("img");
        particle.classList.add("particle");
        particle.src = src;
        html.appendChild(particle);
    };
    for (let _ = 0; _ < 25; _++) {
        add_particle();
    }
    src = "https://static.igem.wiki/teams/4161/wiki/ambient2.png";
    for (let _ = 0; _ < 25; _++) {
        add_particle();
    }
    const update_ambient = () => {
        const images = document.getElementsByClassName("particle");
        for (let _ = 0; _ < x.length && _ < images.length; _++) {
            const image = images[_];
            x[_] += x_speed[_];
            y[_] += y_speed[_];
            r[_] += r_speed[_];
            image.style.left = (x[_] % window.innerWidth) + "px";
            image.style.top = (y[_] % window.innerHeight) + "px";
            image.style.transform = `rotate(${r[_]}deg)`;
        }
        setTimeout(update_ambient, 30);
    };
    update_ambient();
};

const snap_home_articles = () => {
    let snapping = false;
    let current_scroll = window.scrollY;
    let scroll_up = true;
    const articles = document.querySelectorAll(
        "main > article, #promotion_video"
    );
    const first_article = articles[0];
    const last_article = articles[articles.length - 1];
    const window_middle = () => window.innerHeight / 2 + window.scrollY;
    const article_middle = (article) =>
        article.offsetHeight / 2 + article.offsetTop;
    const scroll_to = (article) =>
        window.scrollTo(0, article_middle(article) - window.innerHeight / 2);
    const snap = () => {
        scroll_up = window.scrollY < current_scroll;
        current_scroll = window.scrollY;
        if (
            snapping ||
            window.innerWidth < 640 ||
            window.innerHeight + 200 < last_article.offsetHeight
        ) {
            return;
        }
        snapping = true;
        if (
            window_middle() <= article_middle(first_article) ||
            window_middle() >= article_middle(last_article)
        ) {
            return;
        }
        let index = 0;
        let article = last_article;
        for (; index < articles.length; index++) {
            const relative_y =
                article_middle(articles[index]) - window_middle();
            if (relative_y >= 0) {
                article = scroll_up ? articles[index - 1] : articles[index];
                break;
            }
        }
        scroll_to(article);
    };
    const not_snapping = () => (snapping = false);
    return [snap, not_snapping];
};

const rotate_home_logo = () => {
    const logo = document.getElementById("home_logo");
    if (logo === null) {
        return () => 0;
    }
    let timer = null;
    let old_offset = window.scrollY;
    const rotate = (offset) => (logo.style.transform = `rotate(${offset}deg)`);
    const [snap, not_snapping] = snap_home_articles();
    const reset_rotation = () => {
        rotate(0);
        old_offset = window.scrollY;
        not_snapping();
    };
    const apply_rotation = async () => {
        if (timer !== null) {
            clearTimeout(timer);
        }
        snap();
        rotate(window.scrollY - old_offset);
        timer = setTimeout(reset_rotation, 100);
    };
    return apply_rotation;
};

const add_toc = () => {
    const h2s = document.querySelectorAll("h2");
    if (h2s.length < 2) {
        return () => 0;
    }
    const current_page_candidates = document.querySelectorAll(
        ".chapter li.chapter-item.expanded"
    );
    const current_page =
        current_page_candidates[current_page_candidates.length - 1];
    const arrow_div = document.createElement("div");
    arrow_div.textContent = "❱";
    const toggle_anchor = document.createElement("a");
    toggle_anchor.classList.add("toggle");
    toggle_anchor.append(arrow_div);
    toggle_anchor.addEventListener("click", () =>
        current_page.classList.toggle("expanded")
    );
    current_page.append(toggle_anchor);
    const toc_ol = document.createElement("ol");
    toc_ol.classList.add("section");
    const h2_w_toc = [];
    for (const h2 of h2s) {
        const anchor = document.createElement("a");
        anchor.textContent = h2.textContent;
        anchor.href = `#${h2.id}`;
        anchor.classList.add("toc-item");
        const header2 = document.createElement("li");
        header2.classList.add("chapter-item");
        header2.append(anchor);
        h2_w_toc.push([h2, header2]);
        toc_ol.append(header2);
    }
    const toc_li = document.createElement("li");
    toc_li.append(toc_ol);
    current_page.after(toc_li);
    const highlight_current_h2 = async () => {
        let found_current_h2 = false;
        let last_passed_toc = null;
        for (const [h2, toc] of h2_w_toc) {
            if (!found_current_h2) {
                if (window.innerHeight / 3 + window.scrollY > h2.offsetTop) {
                    last_passed_toc = toc;
                } else {
                    found_current_h2 = true;
                    if (last_passed_toc !== null) {
                        last_passed_toc.classList.add("expanded");
                    }
                }
            }
            toc.classList.remove("expanded");
        }
        if (!found_current_h2 && last_passed_toc !== null) {
            // The last h2 is the one to highlight
            last_passed_toc.classList.add("expanded");
        }
    };
    return highlight_current_h2;
};

const hide_loading = () => {
    let main = document.querySelector("main");
    main.style.display = "block";
    let main_loading = document.getElementById("main-loading");
    main_loading.style.display = "none";
};

const fade_in = () => {
    let text_blocks = [];
    for (const block of document.querySelectorAll("main > *")) {
        if (block.style?.opacity !== undefined) {
            text_blocks.push(block);
        }
    }
    const set_all_opacity = async () => {
        for (const block of text_blocks) {
            if (
                window.scrollY <= block.offsetTop + block.offsetHeight &&
                window.scrollY + window.innerHeight >= block.offsetTop
            ) {
                block.style.opacity = 1;
            } else {
                block.style.opacity = 0;
            }
        }
    };
    return set_all_opacity;
};

const scrolling_effect = () => {
    let active = false;
    let scheduled = false;
    const rotate_logo = rotate_home_logo();
    const update_toc = add_toc();
    const fade = fade_in();
    fade();
    const apply = () => {
        scheduled = false;
        update_toc();
        rotate_logo();
        fade();
        if (scheduled) {
            setTimeout(apply, 50);
        } else {
            active = false;
        }
    };
    const schedule = () => {
        if (active) {
            scheduled = true;
        } else {
            active = true;
            setTimeout(apply, 50);
        }
    };
    window.addEventListener("scroll", schedule);
};

const load = () => {
    sidebar_aria();
    codeSnippets();
    themes();
    sidebar();
    chapterNavigation();
    clipboard();
    scrollToTop();
    controllMenu();
    create_ambient();
    hide_loading();
    scrolling_effect();
};

window.addEventListener("load", load);

// Global variable, shared between modules
function playground_text(playground) {
    let code_block = playground.querySelector("code");

    if (window.ace && code_block.classList.contains("editable")) {
        let editor = window.ace.edit(code_block);
        return editor.getValue();
    } else {
        return code_block.textContent;
    }
}

function codeSnippets() {
    function fetch_with_timeout(url, options, timeout = 6000) {
        return Promise.race([
            fetch(url, options),
            new Promise((_, reject) =>
                setTimeout(() => reject(new Error("timeout")), timeout)
            ),
        ]);
    }

    var playgrounds = Array.from(document.querySelectorAll(".playground"));
    if (playgrounds.length > 0) {
        fetch_with_timeout("https://play.rust-lang.org/meta/crates", {
            headers: {
                "Content-Type": "application/json",
            },
            method: "POST",
            mode: "cors",
        })
            .then((response) => response.json())
            .then((response) => {
                // get list of crates available in the rust playground
                let playground_crates = response.crates.map(
                    (item) => item["id"]
                );
                playgrounds.forEach((block) =>
                    handle_crate_list_update(block, playground_crates)
                );
            });
    }

    function handle_crate_list_update(playground_block, playground_crates) {
        // update the play buttons after receiving the response
        update_play_button(playground_block, playground_crates);

        // and install on change listener to dynamically update ACE editors
        if (window.ace) {
            let code_block = playground_block.querySelector("code");
            if (code_block.classList.contains("editable")) {
                let editor = window.ace.edit(code_block);
                editor.addEventListener("change", function (e) {
                    update_play_button(playground_block, playground_crates);
                });
                // add Ctrl-Enter command to execute rust code
                editor.commands.addCommand({
                    name: "run",
                    bindKey: {
                        win: "Ctrl-Enter",
                        mac: "Ctrl-Enter",
                    },
                    exec: (_editor) => run_rust_code(playground_block),
                });
            }
        }
    }

    // updates the visibility of play button based on `no_run` class and
    // used crates vs ones available on http://play.rust-lang.org
    function update_play_button(pre_block, playground_crates) {
        var play_button = pre_block.querySelector(".play-button");

        // skip if code is `no_run`
        if (pre_block.querySelector("code").classList.contains("no_run")) {
            play_button.classList.add("hidden");
            return;
        }

        // get list of `extern crate`'s from snippet
        var txt = playground_text(pre_block);
        var re = /extern\s+crate\s+([a-zA-Z_0-9]+)\s*;/g;
        var snippet_crates = [];
        var item;
        while ((item = re.exec(txt))) {
            snippet_crates.push(item[1]);
        }

        // check if all used crates are available on play.rust-lang.org
        var all_available = snippet_crates.every(function (elem) {
            return playground_crates.indexOf(elem) > -1;
        });

        if (all_available) {
            play_button.classList.remove("hidden");
        } else {
            play_button.classList.add("hidden");
        }
    }

    function run_rust_code(code_block) {
        var result_block = code_block.querySelector(".result");
        if (!result_block) {
            result_block = document.createElement("code");
            result_block.className = "result hljs language-bash";

            code_block.append(result_block);
        }

        let text = playground_text(code_block);
        let classes = code_block.querySelector("code").classList;
        let edition = "2015";
        if (classes.contains("edition2018")) {
            edition = "2018";
        } else if (classes.contains("edition2021")) {
            edition = "2021";
        }
        var params = {
            version: "stable",
            optimize: "0",
            code: text,
            edition: edition,
        };

        if (text.indexOf("#![feature") !== -1) {
            params.version = "nightly";
        }

        result_block.innerText = "Running...";

        fetch_with_timeout("https://play.rust-lang.org/evaluate.json", {
            headers: {
                "Content-Type": "application/json",
            },
            method: "POST",
            mode: "cors",
            body: JSON.stringify(params),
        })
            .then((response) => response.json())
            .then((response) => {
                if (response.result.trim() === "") {
                    result_block.innerText = "No output";
                    result_block.classList.add("result-no-output");
                } else {
                    result_block.innerText = response.result;
                    result_block.classList.remove("result-no-output");
                }
            })
            .catch(
                (error) =>
                    (result_block.innerText =
                        "Playground Communication: " + error.message)
            );
    }

    // Syntax highlighting Configuration
    hljs.configure({
        tabReplace: "    ", // 4 spaces
        languages: [], // Languages used for auto-detection
    });

    let code_nodes = Array.from(document.querySelectorAll("code"))
        // Don't highlight `inline code` blocks in headers.
        .filter(function (node) {
            return !node.parentElement.classList.contains("header");
        });

    if (window.ace) {
        // language-rust class needs to be removed for editable
        // blocks or highlightjs will capture events
        code_nodes
            .filter(function (node) {
                return node.classList.contains("editable");
            })
            .forEach(function (block) {
                block.classList.remove("language-rust");
            });

        Array;
        code_nodes
            .filter(function (node) {
                return !node.classList.contains("editable");
            })
            .forEach(function (block) {
                hljs.highlightBlock(block);
            });
    } else {
        code_nodes.forEach(function (block) {
            hljs.highlightBlock(block);
        });
    }

    // Adding the hljs class gives code blocks the color css
    // even if highlighting doesn't apply
    code_nodes.forEach(function (block) {
        block.classList.add("hljs");
    });

    Array.from(document.querySelectorAll("code.language-rust")).forEach(
        function (block) {
            var lines = Array.from(block.querySelectorAll(".boring"));
            // If no lines were hidden, return
            if (!lines.length) {
                return;
            }
            block.classList.add("hide-boring");

            var buttons = document.createElement("div");
            buttons.className = "buttons";
            buttons.innerHTML =
                '<button class="fa fa-eye" title="Show hidden lines" aria-label="Show hidden lines"></button>';

            // add expand button
            var pre_block = block.parentNode;
            pre_block.insertBefore(buttons, pre_block.firstChild);

            pre_block
                .querySelector(".buttons")
                .addEventListener("click", function (e) {
                    if (e.target.classList.contains("fa-eye")) {
                        e.target.classList.remove("fa-eye");
                        e.target.classList.add("fa-eye-slash");
                        e.target.title = "Hide lines";
                        e.target.setAttribute("aria-label", e.target.title);

                        block.classList.remove("hide-boring");
                    } else if (e.target.classList.contains("fa-eye-slash")) {
                        e.target.classList.remove("fa-eye-slash");
                        e.target.classList.add("fa-eye");
                        e.target.title = "Show hidden lines";
                        e.target.setAttribute("aria-label", e.target.title);

                        block.classList.add("hide-boring");
                    }
                });
        }
    );

    if (window.playground_copyable) {
        Array.from(document.querySelectorAll("pre code")).forEach(function (
            block
        ) {
            var pre_block = block.parentNode;
            if (!pre_block.classList.contains("playground")) {
                var buttons = pre_block.querySelector(".buttons");
                if (!buttons) {
                    buttons = document.createElement("div");
                    buttons.className = "buttons";
                    pre_block.insertBefore(buttons, pre_block.firstChild);
                }

                var clipButton = document.createElement("button");
                clipButton.className = "fa fa-copy clip-button";
                clipButton.title = "Copy to clipboard";
                clipButton.setAttribute("aria-label", clipButton.title);
                clipButton.innerHTML = '<i class="tooltiptext"></i>';

                buttons.insertBefore(clipButton, buttons.firstChild);
            }
        });
    }

    // Process playground code blocks
    Array.from(document.querySelectorAll(".playground")).forEach(function (
        pre_block
    ) {
        // Add play button
        var buttons = pre_block.querySelector(".buttons");
        if (!buttons) {
            buttons = document.createElement("div");
            buttons.className = "buttons";
            pre_block.insertBefore(buttons, pre_block.firstChild);
        }

        var runCodeButton = document.createElement("button");
        runCodeButton.className = "fa fa-play play-button";
        runCodeButton.hidden = true;
        runCodeButton.title = "Run this code";
        runCodeButton.setAttribute("aria-label", runCodeButton.title);

        buttons.insertBefore(runCodeButton, buttons.firstChild);
        runCodeButton.addEventListener("click", function (e) {
            run_rust_code(pre_block);
        });

        if (window.playground_copyable) {
            var copyCodeClipboardButton = document.createElement("button");
            copyCodeClipboardButton.className = "fa fa-copy clip-button";
            copyCodeClipboardButton.innerHTML = '<i class="tooltiptext"></i>';
            copyCodeClipboardButton.title = "Copy to clipboard";
            copyCodeClipboardButton.setAttribute(
                "aria-label",
                copyCodeClipboardButton.title
            );

            buttons.insertBefore(copyCodeClipboardButton, buttons.firstChild);
        }

        let code_block = pre_block.querySelector("code");
        if (window.ace && code_block.classList.contains("editable")) {
            var undoChangesButton = document.createElement("button");
            undoChangesButton.className = "fa fa-history reset-button";
            undoChangesButton.title = "Undo changes";
            undoChangesButton.setAttribute(
                "aria-label",
                undoChangesButton.title
            );

            buttons.insertBefore(undoChangesButton, buttons.firstChild);

            undoChangesButton.addEventListener("click", function () {
                let editor = window.ace.edit(code_block);
                editor.setValue(editor.originalCode);
                editor.clearSelection();
            });
        }
    });
}

function themes() {
    var themeToggleButton = document.getElementById("theme-toggle");
    var themePopup = document.getElementById("theme-list");
    var themeColorMetaTag = document.querySelector('meta[name="theme-color"]');
    var stylesheets = {
        ayuHighlight: document.querySelector("[href$='ayu-highlight.css']"),
        tomorrowNight: document.querySelector("[href$='tomorrow-night.css']"),
        highlight: document.querySelector("[href$='highlight.css']"),
    };

    function showThemes() {
        themePopup.style.display = "block";
        themeToggleButton.setAttribute("aria-expanded", true);
        themePopup.querySelector("button#" + get_theme()).focus();
    }

    function hideThemes() {
        themePopup.style.display = "none";
        themeToggleButton.setAttribute("aria-expanded", false);
        themeToggleButton.focus();
    }

    function get_theme() {
        var theme;
        try {
            theme = localStorage.getItem("mdbook-theme");
        } catch (e) {}
        if (theme === null || theme === undefined) {
            return default_theme;
        } else {
            return theme;
        }
    }

    function set_theme(theme, store = true) {
        let ace_theme;

        if (theme == "coal" || theme == "navy") {
            stylesheets.ayuHighlight.disabled = true;
            stylesheets.tomorrowNight.disabled = false;
            stylesheets.highlight.disabled = true;

            ace_theme = "ace/theme/tomorrow_night";
        } else if (theme == "ayu") {
            stylesheets.ayuHighlight.disabled = false;
            stylesheets.tomorrowNight.disabled = true;
            stylesheets.highlight.disabled = true;
            ace_theme = "ace/theme/tomorrow_night";
        } else {
            stylesheets.ayuHighlight.disabled = true;
            stylesheets.tomorrowNight.disabled = true;
            stylesheets.highlight.disabled = false;
            ace_theme = "ace/theme/dawn";
        }

        setTimeout(function () {
            themeColorMetaTag.content = getComputedStyle(
                document.body
            ).backgroundColor;
        }, 1);

        if (window.ace && window.editors) {
            window.editors.forEach(function (editor) {
                editor.setTheme(ace_theme);
            });
        }

        var previousTheme = get_theme();

        if (store) {
            try {
                localStorage.setItem("mdbook-theme", theme);
            } catch (e) {}
        }

        html.classList.remove(previousTheme);
        html.classList.add(theme);
    }

    // Set theme
    var theme = get_theme();

    set_theme(theme, false);

    themeToggleButton.addEventListener("click", function () {
        if (themePopup.style.display === "block") {
            hideThemes();
        } else {
            showThemes();
        }
    });

    themePopup.addEventListener("click", function (e) {
        var theme;
        if (e.target.className === "theme") {
            theme = e.target.id;
        } else if (e.target.parentElement.className === "theme") {
            theme = e.target.parentElement.id;
        } else {
            return;
        }
        set_theme(theme);
    });

    themePopup.addEventListener("focusout", function (e) {
        // e.relatedTarget is null in Safari and Firefox on macOS (see workaround below)
        if (
            !!e.relatedTarget &&
            !themeToggleButton.contains(e.relatedTarget) &&
            !themePopup.contains(e.relatedTarget)
        ) {
            hideThemes();
        }
    });

    // Should not be needed, but it works around an issue on macOS & iOS: https://github.com/rust-lang/mdBook/issues/628
    document.addEventListener("click", function (e) {
        if (
            themePopup.style.display === "block" &&
            !themeToggleButton.contains(e.target) &&
            !themePopup.contains(e.target)
        ) {
            hideThemes();
        }
    });

    document.addEventListener("keydown", function (e) {
        if (e.altKey || e.ctrlKey || e.metaKey || e.shiftKey) {
            return;
        }
        if (!themePopup.contains(e.target)) {
            return;
        }

        switch (e.key) {
            case "Escape":
                e.preventDefault();
                hideThemes();
                break;
            case "ArrowUp":
                e.preventDefault();
                var li = document.activeElement.parentElement;
                if (li && li.previousElementSibling) {
                    li.previousElementSibling.querySelector("button").focus();
                }
                break;
            case "ArrowDown":
                e.preventDefault();
                var li = document.activeElement.parentElement;
                if (li && li.nextElementSibling) {
                    li.nextElementSibling.querySelector("button").focus();
                }
                break;
            case "Home":
                e.preventDefault();
                themePopup.querySelector("li:first-child button").focus();
                break;
            case "End":
                e.preventDefault();
                themePopup.querySelector("li:last-child button").focus();
                break;
        }
    });
}

function sidebar() {
    var html = document.querySelector("html");
    var sidebar = document.getElementById("sidebar");
    var sidebarLinks = document.querySelectorAll("#sidebar a");
    var sidebarToggleButton = document.getElementById("sidebar-toggle");
    var sidebarResizeHandle = document.getElementById("sidebar-resize-handle");
    var firstContact = null;

    function showSidebar() {
        html.classList.remove("sidebar-hidden");
        html.classList.add("sidebar-visible");
        Array.from(sidebarLinks).forEach(function (link) {
            link.setAttribute("tabIndex", 0);
        });
        sidebarToggleButton.setAttribute("aria-expanded", true);
        sidebar.setAttribute("aria-hidden", false);
        try {
            localStorage.setItem("mdbook-sidebar", "visible");
        } catch (e) {}
    }

    var sidebarAnchorToggles = document.querySelectorAll("#sidebar a.toggle");

    function toggleSection(ev) {
        ev.currentTarget.parentElement.classList.toggle("expanded");
    }

    Array.from(sidebarAnchorToggles).forEach(function (el) {
        el.addEventListener("click", toggleSection);
    });

    function hideSidebar() {
        html.classList.remove("sidebar-visible");
        html.classList.add("sidebar-hidden");
        Array.from(sidebarLinks).forEach(function (link) {
            link.setAttribute("tabIndex", -1);
        });
        sidebarToggleButton.setAttribute("aria-expanded", false);
        sidebar.setAttribute("aria-hidden", true);
        try {
            localStorage.setItem("mdbook-sidebar", "hidden");
        } catch (e) {}
    }

    // Toggle sidebar
    sidebarToggleButton.addEventListener("click", function sidebarToggle() {
        if (html.classList.contains("sidebar-hidden")) {
            var current_width = parseInt(
                document.documentElement.style.getPropertyValue(
                    "--sidebar-width"
                ),
                10
            );
            if (current_width < 150) {
                document.documentElement.style.setProperty(
                    "--sidebar-width",
                    "150px"
                );
            }
            showSidebar();
        } else if (html.classList.contains("sidebar-visible")) {
            hideSidebar();
        } else {
            if (getComputedStyle(sidebar)["transform"] === "none") {
                hideSidebar();
            } else {
                showSidebar();
            }
        }
    });

    sidebarResizeHandle.addEventListener("mousedown", initResize, false);

    function initResize(e) {
        window.addEventListener("mousemove", resize, false);
        window.addEventListener("mouseup", stopResize, false);
        html.classList.add("sidebar-resizing");
    }
    function resize(e) {
        var pos = e.clientX - sidebar.offsetLeft;
        if (pos < 20) {
            hideSidebar();
        } else {
            if (html.classList.contains("sidebar-hidden")) {
                showSidebar();
            }
            pos = Math.min(pos, window.innerWidth - 100);
            document.documentElement.style.setProperty(
                "--sidebar-width",
                pos + "px"
            );
        }
    }
    //on mouseup remove windows functions mousemove & mouseup
    function stopResize(e) {
        html.classList.remove("sidebar-resizing");
        window.removeEventListener("mousemove", resize, false);
        window.removeEventListener("mouseup", stopResize, false);
    }

    document.addEventListener(
        "touchstart",
        function (e) {
            firstContact = {
                x: e.touches[0].clientX,
                time: Date.now(),
            };
        },
        { passive: true }
    );

    document.addEventListener(
        "touchmove",
        function (e) {
            if (!firstContact) return;

            var curX = e.touches[0].clientX;
            var xDiff = curX - firstContact.x,
                tDiff = Date.now() - firstContact.time;

            if (tDiff < 250 && Math.abs(xDiff) >= 150) {
                if (
                    xDiff >= 0 &&
                    firstContact.x <
                        Math.min(document.body.clientWidth * 0.25, 300)
                )
                    showSidebar();
                else if (xDiff < 0 && curX < 300) hideSidebar();

                firstContact = null;
            }
        },
        { passive: true }
    );

    // Scroll sidebar to current active section
    var activeSection = document
        .getElementById("sidebar")
        .querySelector(".active");
    if (activeSection) {
        // https://developer.mozilla.org/en-US/docs/Web/API/Element/scrollIntoView
        activeSection.scrollIntoView({ block: "center" });
    }
}

function chapterNavigation() {
    document.addEventListener("keydown", function (e) {
        if (e.altKey || e.ctrlKey || e.metaKey || e.shiftKey) {
            return;
        }
        if (window.search && window.search.hasFocus()) {
            return;
        }

        switch (e.key) {
            case "ArrowRight":
                e.preventDefault();
                var nextButton = document.querySelector(".nav-chapters.next");
                if (nextButton) {
                    window.location.href = nextButton.href;
                }
                break;
            case "ArrowLeft":
                e.preventDefault();
                var previousButton = document.querySelector(
                    ".nav-chapters.previous"
                );
                if (previousButton) {
                    window.location.href = previousButton.href;
                }
                break;
        }
    });
}

function clipboard() {
    var clipButtons = document.querySelectorAll(".clip-button");

    function hideTooltip(elem) {
        elem.firstChild.innerText = "";
        elem.className = "fa fa-copy clip-button";
    }

    function showTooltip(elem, msg) {
        elem.firstChild.innerText = msg;
        elem.className = "fa fa-copy tooltipped";
    }

    var clipboardSnippets = new ClipboardJS(".clip-button", {
        text: function (trigger) {
            hideTooltip(trigger);
            let playground = trigger.closest("pre");
            return playground_text(playground);
        },
    });

    Array.from(clipButtons).forEach(function (clipButton) {
        clipButton.addEventListener("mouseout", function (e) {
            hideTooltip(e.currentTarget);
        });
    });

    clipboardSnippets.on("success", function (e) {
        e.clearSelection();
        showTooltip(e.trigger, "Copied!");
    });

    clipboardSnippets.on("error", function (e) {
        showTooltip(e.trigger, "Clipboard error!");
    });
}

function scrollToTop() {
    var menuTitle = document.querySelector(".menu-title");

    menuTitle.addEventListener("click", function () {
        document.scrollingElement.scrollTo({ top: 0, behavior: "smooth" });
    });
}

function controllMenu() {
    var menu = document.getElementById("menu-bar");

    (function controllPosition() {
        var scrollTop = document.scrollingElement.scrollTop;
        var prevScrollTop = scrollTop;
        var minMenuY = -menu.clientHeight - 50;
        // When the script loads, the page can be at any scroll (e.g. if you reforesh it).
        menu.style.top = scrollTop + "px";
        // Same as parseInt(menu.style.top.slice(0, -2), but faster
        var topCache = menu.style.top.slice(0, -2);
        menu.classList.remove("sticky");
        var stickyCache = false; // Same as menu.classList.contains('sticky'), but faster
        document.addEventListener(
            "scroll",
            function () {
                scrollTop = Math.max(document.scrollingElement.scrollTop, 0);
                // `null` means that it doesn't need to be updated
                var nextSticky = null;
                var nextTop = null;
                var scrollDown = scrollTop > prevScrollTop;
                var menuPosAbsoluteY = topCache - scrollTop;
                if (scrollDown) {
                    nextSticky = false;
                    if (menuPosAbsoluteY > 0) {
                        nextTop = prevScrollTop;
                    }
                } else {
                    if (menuPosAbsoluteY > 0) {
                        nextSticky = true;
                    } else if (menuPosAbsoluteY < minMenuY) {
                        nextTop = prevScrollTop + minMenuY;
                    }
                }
                if (nextSticky === true && stickyCache === false) {
                    menu.classList.add("sticky");
                    stickyCache = true;
                } else if (nextSticky === false && stickyCache === true) {
                    menu.classList.remove("sticky");
                    stickyCache = false;
                }
                if (nextTop !== null) {
                    menu.style.top = nextTop + "px";
                    topCache = nextTop;
                }
                prevScrollTop = scrollTop;
            },
            { passive: true }
        );
    })();
    (function controllBorder() {
        menu.classList.remove("bordered");
        document.addEventListener(
            "scroll",
            function () {
                if (menu.offsetTop === 0) {
                    menu.classList.remove("bordered");
                } else {
                    menu.classList.add("bordered");
                }
            },
            { passive: true }
        );
    })();
}
